/* 
 * CSE142, Spring 1999
 * Homework 4
 * Traffic animation

  STARTER PROGRAM

 * Section AE
 *
 */

#include "gp142.h"
#include <stdio.h>
#include <math.h>

#include <time.h>
	/* 
	* <time.h> is included so the current time can be accessed
	* to initialize the random number generator.  If you don't
	* use random numbers (and you don't have to) you can either
	* ignore this #include or delete it.
	*/

/* Logical constants */
#define TRUE    		1
#define FALSE   		0

/* 
 * For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.  The
 * constant FILL is used in function calls when a solid
 * shape is desired.
 */
#define FILL 0

/*
 * Symbolic constants.  Change as needed.
 */

/* Menu buttons: */

/* Top-left coordinates of button palette */
#define MENU_TOP		(GP142_YMAX - 5)
#define MENU_LEFT		(-GP142_XMAX + 5)

/* Height and width of buttons in pixels */
#define BUTTON_HEIGHT  	25
#define BUTTON_WIDTH 	75

/* Number of buttons */
#define NUM_BUTTONS 	8

/* Possible locations of mouse clicks: */

/*   Mouse click in some button */
#define BUTTON_CLICK 1

/*   Mouse click elsewhere */
#define OTHER_CLICK 0

/* Coordinates of objects in the drawing */

/* y-coordinate of the top of the ground in the landscape */
#define HORIZON (- 2 * (GP142_YMAX / 10))

/* Vertical center and width of highway */
#define HIGHWAY_CENTER (HORIZON + 60)
#define HIGHWAY_WIDTH  10

/* Maximum number of vehicles that can appear in the animation */
#define MAX_CARS	75

/* Height and width of a single car */
#define CAR_HEIGHT 12
#define CAR_WIDTH  18

typedef struct{
        int x, y;		/* current position of the car					*/
	    int direction;	/* current direction: +1=right, -1=left			*/
	    int color;		/* color of the car								*/
	    int speed;		/* speed in pixels per tick when traffic moving	*/	
	    int moving;		/* = "traffic is currently moving"				*/
    } car;

/*****************************************************************************

     FUNCTION PROTOTYPES

*****************************************************************************/

/* Initialize random number generator.  [Ignore or		*/
/* remove if your program doesn't use random numbers.]	*/
void initialize_random_numbers();

/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y, int boat_x, int boat_y);

//Draw windows
void draw_windows(int max_count, int x1, int y1, int max_x1, int color_windows);

//Draw arched windows
void draw_arched_windows(int max_count, int x1, int y1, int max_x1, int color_windows);

//Draw dashed street lines
void draw_dashed_line(int y);

//Draw bridge cables
void draw_bridge_cables();

/* Draw the buttons */
void draw_buttons();

/* Return kind of mouse click (in a button, etc.) */
int  classify_mouse_click(int mouse_x, int mouse_y);

/* Process button click at location x, y.  Update car speed,  */
/* direction, color, and whether traffic is moving as needed. */
void handle_button(int x, int y, car traffic[], int *i);

/* Update sun and boat coordinates for specified ticks */
void move_sun_and_boat(int *sun_x, int *sun_y, int *boat_x, int *boat_y);

/* Update all vehicle x, y coordinates, given current x, y */
void move_traffic(car traffic[], int *i);

/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(car traffic[], int i);

/* Display error message if internal error is detected. */
void something_is_wrong (void);

/*****************************************************************************

     MAIN

*****************************************************************************/

/*
 * Main program contents:
 *   - Major program variables that persist throughout execution
 *   - Event loop that processes GP142 events until the user quits,
 *     and updates variables and redraws the window as needed.
 */
int main(void)
{
	/* Event handling and buttons: */
	int  quit;				/* = "user has selected quit"				*/
	int  event;				/* the most recent GP142 event				*/
	char key_pressed;		/* last keyboard character from key event	*/
	int  mouse_x, mouse_y;	/* x and y coordinates of latest mouse event*/
	int  click_kind;		/* kind of mouse click (button, etc.)		*/

	/* Information about the sun and boat: */
	int sun_x, sun_y;			/* screen x and y coordinates */
    int boat_x, boat_y;         //two screen x and y coordinates

	/* Information about vehicles in the animation:							*/
	
    car traffic[MAX_CARS];
    int i;  //which car 
    int j;  //counter active cars
    
    initialize_random_numbers();
	
	/* Initialize graphics package, clear the window,	*/
	/* turn logging off, and start animation.			*/
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);

	/* set initial locations of sun and boat */
	sun_x = GP142_XMAX - 50;
	sun_y = GP142_YMAX - 35;
    boat_x = -GP142_XMAX;
    boat_y = -GP142_YMAX;
	
    /* set initial conditions for the car */
    for ( i = 0; i < MAX_CARS; i++ ){
        if ( i % 4 == 0){
            traffic[i].x = -GP142_XMAX - CAR_WIDTH,
            traffic[i].y = HIGHWAY_CENTER - 23;
	        traffic[i].direction = +1;
	        traffic[i].speed = 1;
            traffic[i].color = GREEN;
        }else if( i % 4 == 1){
            traffic[i].x = GP142_XMAX + CAR_WIDTH;
            traffic[i].y = HIGHWAY_CENTER - 5;
	        traffic[i].direction = -1;
	        traffic[i].speed = 1;
            traffic[i].color = BLUE;
        }else if( i % 4 == 2){
            traffic[i].x = GP142_XMAX + CAR_WIDTH;
            traffic[i].y = HIGHWAY_CENTER + 11;
	        traffic[i].direction = -1;
	        traffic[i].speed = 2;
            traffic[i].color = RED;
        }else{
            traffic[i].x = GP142_XMAX + CAR_WIDTH;
            traffic[i].y = HIGHWAY_CENTER -38;
	        traffic[i].direction = +1;
            traffic[i].speed = 2;
            traffic[i].color = MAGENTA;
        }
	        traffic[i].moving = TRUE;
    }
    
    /* Draw initial landscape, buttons, and traffic */
   	draw_landscape(sun_x,sun_y, boat_x, boat_y);
	draw_buttons();
    i = 0;
    draw_car(traffic, i);
    draw_bridge_cables();
    
    /*
	 * Main event loop:
	 * ---- ----- -----
	 * Wait for the next user action, decode it, and call an
	 * appropriate function to handle it.  Repeat until the
	 * user selects quit.
	 */
	quit = FALSE;
	while (!quit) {
		
		/* get next event */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_pressed);
		
		/* perform appropriate action depending on kind of event */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected */
			quit = TRUE;
			break;
			
		case GP142_KBD:
			/* Key pressed.  Not used now.  If the program should		*/
			/* react to keyboard events, insert something appropriate	*/
			/* here.  Note that scanf CANNOT be used with GP142.		*/
			break;
			
		case GP142_MOUSE:
			/* Mouse click.  Determine if in a menu button	*/
			/* or elsewhere and process accordingly.		*/
			click_kind = classify_mouse_click(mouse_x, mouse_y);
			if (click_kind == BUTTON_CLICK) {
                handle_button(mouse_x, mouse_y, traffic, &i);
			} else {
				/* insert code here (and additional else-if statements) */
				/* to react to mouse clicks elsewhere on the screen */
			}
			break;
			
		case GP142_PERIODIC:
			/* Timer interval event.  Update everything	*/
			/* that needs to change as time advances.	*/
			move_sun_and_boat(&sun_x, &sun_y, &boat_x, &boat_y);
                       
            for ( j = 0; j <= i; j++ ){
			if (traffic[j].moving)
				move_traffic(traffic, &i);
			}
            break;
			
		default:
			/* unknown event */
			break;
			
		}  /*end switch */
		
		/* Redraw everything */
        draw_landscape(sun_x, sun_y, boat_x, boat_y);
		draw_buttons();
        for ( j = 0; j < i; j++ ) draw_car(traffic, j);
        draw_bridge_cables();
       
    }  /* end event loop */
	
	/* Termination: shut down graphics window and exit */
	GP142_close();
	return 0;
}  /*end main */

/*****************************************************************************

     BACKGROUND LANDSCAPE

*****************************************************************************/

/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y, int boat_x, int boat_y)
{
    int text_offset = -GP142_XMAX + 15;	/* horizontal offset of text */
	int text_size   = 12;				/* size of text font */
    int color_sky;
    int color_sun;
    int color_windows;
    int c;

    if (sun_x <= -GP142_XMAX - 30){
        color_sky = BLACK;
        color_sun = WHITE;
        color_windows = YELLOW + rand() % 3;
    }else{
        color_sky = ORANGE;
        color_sun = YELLOW;
        color_windows = BLACK;
    }
        
    /* Draw the sky */
	GP142_rectangleXY(color_sky, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, GP142_YMAX, FILL);
	
	/* Draw the sun */
	GP142_circleXY(color_sun, sun_x, sun_y , 30);
   
    //Draw buildings
	GP142_rectangleXY(WHITE, -GP142_XMAX, HORIZON, -220, GP142_YMAX - 50, FILL); 
	GP142_rectangleXY(MED_GRAY, -215, HORIZON, -145, GP142_YMAX - 15, FILL);
	GP142_rectangleXY(FOREST_GREEN, -140, HORIZON, 10, GP142_YMAX - 100, FILL);
	
	GP142_rectangleXY(PINK, -135, HORIZON, -40, HORIZON + 100, FILL);
	GP142_rectangleXY(ICE_BLUE, 15, HORIZON, 85, HORIZON + 170, FILL);
		GP142_triangleXY(ICE_BLUE, 15, HORIZON + 170, 85, HORIZON + 170, 50, HORIZON + 220, FILL);
		GP142_rectangleXY(ICE_BLUE, 25, HORIZON + 170, 75, HORIZON + 200, FILL);
		GP142_triangleXY(ICE_BLUE, 25, HORIZON + 200, 75, HORIZON + 200, 50, HORIZON + 240, FILL);
		GP142_rectangleXY(ICE_BLUE, 35, HORIZON + 200, 65, HORIZON + 230, FILL);
		GP142_triangleXY(ICE_BLUE, 35, HORIZON + 230, 65, HORIZON + 230, 50, HORIZON + 260, FILL);
		GP142_rectangleXY(ICE_BLUE, 45, HORIZON + 230, 55, HORIZON + 260, FILL);
		GP142_triangleXY(ICE_BLUE, 47, HORIZON + 260, 53, HORIZON + 260, 50, HORIZON + 300, FILL);

 	GP142_rectangleXY(DUSTY_PLUM, 93, HORIZON, 132, GP142_YMAX - 120, FILL);
	GP142_rectangleXY(TURQUOISE, 140, HORIZON, 250, GP142_YMAX - 25, FILL);
	GP142_rectangleXY(MAGENTA, 205, HORIZON, GP142_XMAX, HORIZON + 125, FILL);
	
	//Draw windows
    draw_arched_windows(40, 25, HORIZON + 160, 60, color_windows);
	draw_arched_windows(50, -GP142_XMAX + 3, GP142_YMAX - 70, -245, color_windows);
	draw_arched_windows(40, -125, HORIZON + 85, -70, color_windows);
    draw_windows(45, -205, GP142_YMAX - 25, -170, color_windows);
    draw_windows(28, -135, GP142_YMAX -110, -30, color_windows);
	draw_windows(11, -33, HORIZON + 90, -30, color_windows);
    draw_windows(25, 98, GP142_YMAX - 130, 115, color_windows);
    draw_windows(70, 150, GP142_YMAX - 35, 230, color_windows);
    draw_windows(30, 230, HORIZON + 115, GP142_XMAX, color_windows);

	/* Draw the water, bridge and text legend */
	GP142_rectangleXY(BLUE, -GP142_XMAX, -GP142_YMAX + rand() % 2,
		GP142_XMAX, HORIZON + rand() % 2, FILL);

	GP142_rectangleXY(LT_GRAY, 
					-GP142_XMAX, HIGHWAY_CENTER - (5 * HIGHWAY_WIDTH),
					 GP142_XMAX, HIGHWAY_CENTER + (2 * HIGHWAY_WIDTH),
					 FILL);
	GP142_lineXY(YELLOW, -GP142_XMAX, HIGHWAY_CENTER - 15, GP142_XMAX, HIGHWAY_CENTER - 15, FILL);
	GP142_lineXY(YELLOW, -GP142_XMAX, HIGHWAY_CENTER - 10, GP142_XMAX, HIGHWAY_CENTER - 10, FILL);
	
    draw_dashed_line(HIGHWAY_CENTER + 5);
    draw_dashed_line(HIGHWAY_CENTER - 30);

    GP142_lineXY(GOLD, 0, HIGHWAY_CENTER + 20, 0, HIGHWAY_CENTER + 30, FILL);
    GP142_lineXY(GOLD, 35, HIGHWAY_CENTER + 20, 35, HIGHWAY_CENTER + 35, FILL);
    GP142_lineXY(GOLD, 70, HIGHWAY_CENTER + 20, 70, HIGHWAY_CENTER +  39, FILL);
    GP142_lineXY(GOLD, 105, HIGHWAY_CENTER + 20, 105, HIGHWAY_CENTER + 45, FILL);
    GP142_lineXY(GOLD, 140, HIGHWAY_CENTER + 20, 140, HIGHWAY_CENTER + 55, FILL);
    GP142_lineXY(GOLD, 175, HIGHWAY_CENTER + 20, 175, HIGHWAY_CENTER + 70, FILL);
    GP142_lineXY(GOLD, 210, HIGHWAY_CENTER + 20, 210, HIGHWAY_CENTER + 90, FILL);
    GP142_lineXY(GOLD, 245, HIGHWAY_CENTER + 20, 245, HIGHWAY_CENTER + 115, FILL);
    GP142_lineXY(GOLD, 280, HIGHWAY_CENTER + 20, 280, HIGHWAY_CENTER + 155, FILL);

    GP142_lineXY(GOLD, -0, HIGHWAY_CENTER + 20, -0, HIGHWAY_CENTER + 30, FILL);
    GP142_lineXY(GOLD, -35, HIGHWAY_CENTER + 20, -35, HIGHWAY_CENTER + 35, FILL);
    GP142_lineXY(GOLD, -70, HIGHWAY_CENTER + 20, -70, HIGHWAY_CENTER +  39, FILL);
    GP142_lineXY(GOLD, -105, HIGHWAY_CENTER + 20, -105, HIGHWAY_CENTER + 45, FILL);
    GP142_lineXY(GOLD, -140, HIGHWAY_CENTER + 20, -140, HIGHWAY_CENTER + 55, FILL);
    GP142_lineXY(GOLD, -175, HIGHWAY_CENTER + 20, -175, HIGHWAY_CENTER + 70, FILL);
    GP142_lineXY(GOLD, -210, HIGHWAY_CENTER + 20, -210, HIGHWAY_CENTER + 90, FILL);
    GP142_lineXY(GOLD, -245, HIGHWAY_CENTER + 20, -245, HIGHWAY_CENTER + 115, FILL);
    GP142_lineXY(GOLD, -280, HIGHWAY_CENTER + 20, -280, HIGHWAY_CENTER + 155, FILL);

    GP142_ovalXY(BLACK, -GP142_XMAX - 10, HIGHWAY_CENTER + 30, GP142_XMAX + 10, GP142_YMAX + 220, 3);
    
    GP142_rectangleXY(MED_GRAY, -GP142_XMAX, HIGHWAY_CENTER - (5 * HIGHWAY_WIDTH) + 5, 
        GP142_XMAX, HIGHWAY_CENTER - (5 * HIGHWAY_WIDTH) - 5, FILL);
    
    //Draw the boat
    GP142_triangleXY(WHITE, boat_x + rand() % 3, boat_y + rand() % 3, boat_x + 30 + rand() % 3, boat_y + rand() % 3, 
        boat_x + 15 + rand() % 3, boat_y + 30 + rand() % 3, FILL);
    GP142_triangleXY(WHITE, boat_x + 15, boat_y + 30, boat_x - 30, boat_y + 30, 
        boat_x, boat_y, FILL);    
    GP142_triangleXY(WHITE, boat_x + 15, boat_y + 30, boat_x + 30, boat_y,
        boat_x + 60 + rand() %3 , boat_y + 30, FILL);
    GP142_lineXY(BROWN, boat_x, boat_y - 1, boat_x, boat_y + 31, 5);
    GP142_lineXY(BROWN, boat_x + 30, boat_y - 1, boat_x + 30, boat_y + 31, 5);
    GP142_circleXY(BROWN, boat_x + 15, boat_y + 15, 10);
    GP142_circleXY(ICE_BLUE, boat_x + 15, boat_y + 15, 8);
    GP142_triangleXY(RED, boat_x + 15, boat_y + 30, boat_x + 40, boat_y + 30, 
        boat_x + 15, boat_y + 80, FILL);
    GP142_triangleXY(YELLOW, boat_x + 15, boat_y + 40 + rand() % 3, boat_x + 15, boat_y + 80 + rand() % 3, 
        boat_x - 5, boat_y + 40 + rand() % 4, FILL);
    GP142_lineXY(BLACK, boat_x + 15, boat_y + 30, boat_x + 15, boat_y + 80, 4);
    
    if(sun_x <= -GP142_XMAX - 30){
        for ( c = 0; c < 3000; c += 15){
        GP142_circleXY(YELLOW, -GP142_XMAX + c + rand() % 4, 
            HIGHWAY_CENTER - ( 5 * HIGHWAY_WIDTH), 2);
        }
    }
	GP142_printfXY (BLACK, text_offset, -GP142_YMAX+30, text_size,
		"Click a button to control the car.");
	
} /*end draw_landscape */

//Draw windows
void draw_windows(int max_count, int x1, int y1, int max_x1, int color_windows){

	int x_1, y_1, x_2, y_2, count = 1;
	x_1 = x1;
	y_1 = y1;

	while( count < max_count ){
	x_2 = x_1 + 10 + rand() % 2;
	y_2 = y_1 - 10 + rand() % 2;

	GP142_rectangleXY(color_windows, x_1, y_1, x_2, y_2, FILL);
	
	if(x_1 >= max_x1){ 
		y_1 = y_1 - 25;
		x_1 = x1;
		}
		else x_1 = x_2 + 10;
	count++;
	}
}
	
void draw_arched_windows(int max_count, int x1, int y1, int max_x1, int color_windows){

	int x_1, y_1, x_2, y_2, count = 1;
	x_1 = x1;
	y_1 = y1;

	while( count < max_count ){
	x_2 = x_1 + 10 + rand() % 2;
	y_2 = y_1 - 10 + rand() % 2;

	GP142_rectangleXY(color_windows, x_1, y_1, x_2, y_2, FILL);
	GP142_circleXY(color_windows, x_1 + 5, y_1, 5);

	if(x_1 >= max_x1){ 
		y_1 = y_1 - 35;
		x_1 = x1;
		}
		else x_1 = x_2 + 10;
	count++;
	}
}	

void draw_dashed_line(int y){
	
	int x = -GP142_XMAX;
	
	while (x < GP142_XMAX){
		GP142_lineXY(YELLOW, x, y, x + 5, y, FILL);
		x = x + 10;
	}
}

void draw_bridge_cables(){

GP142_lineXY(GOLD, 0, HIGHWAY_CENTER - 45, 0, HIGHWAY_CENTER - 35, FILL);
GP142_lineXY(GOLD, 35, HIGHWAY_CENTER - 45, 35, HIGHWAY_CENTER - 34, FILL);
GP142_lineXY(GOLD, 70, HIGHWAY_CENTER - 45, 70, HIGHWAY_CENTER - 30, FILL);
GP142_lineXY(GOLD, 105, HIGHWAY_CENTER - 45, 105, HIGHWAY_CENTER - 20, FILL);
GP142_lineXY(GOLD, 140, HIGHWAY_CENTER - 45, 140, HIGHWAY_CENTER - 15, FILL);
GP142_lineXY(GOLD, 175, HIGHWAY_CENTER - 45, 175, HIGHWAY_CENTER, FILL);
GP142_lineXY(GOLD, 210, HIGHWAY_CENTER - 45, 210, HIGHWAY_CENTER + 15, FILL);
GP142_lineXY(GOLD, 245, HIGHWAY_CENTER - 45, 245, HIGHWAY_CENTER + 40, FILL);
GP142_lineXY(GOLD, 280, HIGHWAY_CENTER - 45, 280, HIGHWAY_CENTER + 70, FILL);

GP142_lineXY(GOLD, -35, HIGHWAY_CENTER - 45, -35, HIGHWAY_CENTER - 34, FILL);
GP142_lineXY(GOLD, -70, HIGHWAY_CENTER - 45, -70, HIGHWAY_CENTER - 30, FILL);
GP142_lineXY(GOLD, -105, HIGHWAY_CENTER - 45, -105, HIGHWAY_CENTER - 20, FILL);
GP142_lineXY(GOLD, -140, HIGHWAY_CENTER - 45, -140, HIGHWAY_CENTER - 15, FILL);
GP142_lineXY(GOLD, -175, HIGHWAY_CENTER - 45, -175, HIGHWAY_CENTER, FILL);
GP142_lineXY(GOLD, -210, HIGHWAY_CENTER - 45, -210, HIGHWAY_CENTER + 15, FILL);
GP142_lineXY(GOLD, -245, HIGHWAY_CENTER - 45, -245, HIGHWAY_CENTER + 40, FILL);
GP142_lineXY(GOLD, -280, HIGHWAY_CENTER - 45, -280, HIGHWAY_CENTER + 70, FILL);

GP142_ovalXY(BLACK, -GP142_XMAX - 20, HIGHWAY_CENTER - 35, 
            GP142_XMAX + 20, GP142_YMAX + 140, 3);
}

/*****************************************************************************

     BUTTONS AND MOUSE CLICKS

*****************************************************************************/

/* Draws the buttons */
void draw_buttons()
{
	int i;
	int text_offset = MENU_LEFT+10;	/* offset of button text in button */
	int font_size = 12;				/* size of button text */
	
	/* draw rectangles for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		GP142_rectangleXY(PINK,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */
	i = MENU_TOP - (BUTTON_HEIGHT/2) - (font_size/2);
	GP142_printfXY(WHITE, text_offset, i, font_size, "Go");	
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(WHITE, text_offset, i, font_size, "Stop");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Faster");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Slower");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Reverse");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "New Color");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Add Car");

    i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Remove Car");

}

/* Classify mouse click at (mouse_x,mouse_y) and return kind */
int  classify_mouse_click(int mouse_x, int mouse_y)
{
	if ((mouse_x > MENU_LEFT) &&
		(mouse_x < MENU_LEFT + BUTTON_WIDTH) &&
		(mouse_y < MENU_TOP) &&
		(mouse_y > (MENU_TOP - NUM_BUTTONS*BUTTON_HEIGHT)))
		return BUTTON_CLICK;
	else
		return OTHER_CLICK;
}

/* Process button click at location x, y.  Update car speed,	*/
/* direction, color, and whether traffic is moving as needed.	*/
void handle_button(int x, int y, car traffic[], int *i)
{
	int buttonNum;		/* number of selected button */
	int j;              //counter for active cars

	/* Complain if x coordinate couldn't be inside a button */
	if (x < MENU_LEFT || x > MENU_LEFT + BUTTON_WIDTH) {
		something_is_wrong();
		return;
	}
	
	/* Convert y coordinate to button number from top. */
	buttonNum = (MENU_TOP - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum){
	case 0:
		/* go */
        for ( j = 0; j <= *i; j++ )
		    traffic[j].moving = TRUE;
		break;
	case 1:
		/* stop */
        for ( j = 0; j <= *i; j++ )
		    traffic[j].moving = FALSE;
		break;
		
	case 2:
		/* speed up */
        for ( j = 0; j <= *i; j++ )
            traffic[j].speed = traffic[j].speed + 1;
		break;
	case 3:
		/* slow down */
        for ( j = 0; j <= *i; j++ )
            traffic[j].speed = traffic[j].speed - 1;
		    if (traffic[j].speed < 0)
			    traffic[j].speed = 0;
		break;
	case 4:
		/* reverse */
        for ( j = 0; j <= *i; j++ )
		    traffic[j].direction = - traffic[j].direction;
        for ( j = *i + 1; j < MAX_CARS; j++ )
            traffic[j].direction = - traffic[j].direction;
		break;
	case 5:
		/* change car color */
        for ( j = 0; j <= *i; j++ )
		    traffic[j].color = (traffic[j].color + 1) % MAX_COLORS;
		break;
	case 6: 
		//add cars
        draw_car(traffic, *i);       
        *i = *i + 1;
        if(*i > MAX_CARS) *i = MAX_CARS - 1;
        break;
    case 7:
        //remove cars
        if (*i > 0) 
            *i -= 1;
        else 
            *i = 0;
        break;
    default:
		/*Bad news if this is ever reached! */
       	something_is_wrong();
		break;
	}
}

/*****************************************************************************

     TRAFFIC AND OTHER ANIMATED OBJECTS

*****************************************************************************/

/* Update sun coordinates sun_x, sun_y by one tick */
void move_sun_and_boat(int *sun_x, int *sun_y, int *boat_x, int *boat_y) {
	/* Move the sun to the left.  If it reaches the edge, move it */
	/* to the other side and down a bit. */

    (*sun_x) -= 4;
    if (*sun_x < -GP142_XMAX - 300) {
		*sun_x = - *sun_x - 300;
		(*sun_y) --;
 	}
    
    *boat_x +=5;

    if (*boat_x < -150 || (*boat_x > 0 && *boat_x < 150))
        *boat_y += 3;
    else *boat_y -= 2;
    
    if (*boat_x > GP142_XMAX + 60) *boat_x = -*boat_x;
    if (*boat_y > HORIZON - 80) *boat_y = -GP142_YMAX;

}

/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick, */
void move_traffic(car traffic[], int *i) {
	
    int trial_x;		/* new value for x coordinate */
    int j;

    for ( j = 0; j < *i; j++ ){

	/* move vehicle.  If it runs off the edge it returns from the other side*/
	trial_x = traffic[j].x + (traffic[j].direction * traffic[j].speed);
	
	if (abs(trial_x) > GP142_XMAX + CAR_WIDTH) traffic[j].x = -(traffic[j].x);
		else traffic[j].x = trial_x;
	}
}

/* Draw vehicles given their current x, y coordinates and color */
void draw_car(car traffic[], int i){
 //   int j;
   // for ( j = 0; j <= i; j++ ) {
    GP142_rectangleXY(traffic[i].color, traffic[i].x - CAR_WIDTH/2, traffic[i].y + CAR_HEIGHT/4,
						traffic[i].x + CAR_WIDTH/2, traffic[i].y + CAR_HEIGHT*3/4, FILL);
	GP142_rectangleXY(traffic[i].color, traffic[i].x - CAR_WIDTH/6, traffic[i].y + CAR_HEIGHT*3/4,
						traffic[i].x + CAR_WIDTH/6, traffic[i].y + CAR_HEIGHT, FILL);
	GP142_circleXY(BLACK, traffic[i].x - CAR_WIDTH/4, traffic[i].y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	GP142_circleXY(BLACK, traffic[i].x + CAR_WIDTH/4, traffic[i].y + CAR_HEIGHT/4, CAR_HEIGHT/4);

}

/*****************************************************************************

     OTHER

*****************************************************************************/

/* Initialize random number generator */
/* [Ignore or delete if your program  */
/* doesn't user random numbers.] */
void initialize_random_numbers() {
	time_t t;			/* current time */
	srand (time(&t));
}

/* Display error message if internal error is detected. */
void something_is_wrong (void) {
	GP142_printfXY (RED, 0, 0, 16, "BAAAADDD NEWS!");
}